#!/usr/bin/perl
#
# Jonathan Klassen
# v1.21: October 17, 2012
#
# Antibase_mass_searcher.pl
#
# Identifies Antibase matches to a list of molecular masses
#
# See the associated README file for copyright license.

use warnings;
use Getopt::Long;

######################################################################################################################
# Processes input arguments
######################################################################################################################

$usage = "
Antibase_mass_searcher_v1.21.pl

USAGE:
-a	column in the input mass list containing the masses (counting from 1) 	DEFAULT: 1	e.g., perl Antibase_mass_searcher.pl -i masses.list -a 8
-b	number of decimal places included in the mass list >=1		DEFAULT: 3		e.g., perl Antibase_mass_searcher.pl -i masses.list -b 2
-c	which Antibase fields to display, in the format \"<field-1>, ..., <field-n>\" - see below for allowable fields

		DEFAULT: \"Name, Sources, Activity, StructCalc\"	e.g., perl Antibase_mass_searcher.pl -i masses.list -c \"Name, StructCalc\"

-d	input Antibase table file					DEFAULT: Antibase.table	e.g., perl Antibase_mass_searcher.pl -i masses.list -d other.table

-e	which Antibase taxa to display, in the format \"<taxon-1>, ..., <taxon-n>\" - see below for allowable taxon names
		DEFAULT: \"All\"					e.g., perl Antibase_mass_searcher.pl -i masses.list -e \"B, C, M\"

-f	mass ions to consider, in the format \"<ion-1>, ..., <ion-n>\" - see below for allowable ion names
		DEFAULT: \"M+, MH+, MNa+\"					e.g., perl Antibase_mass_searcher.pl -i masses.list -f \"M-, MH-\"

-g	output file of unique masses and their IDs			
		DEFAULT: Antibase_IDs.table				e.g., perl Antibase_mass_searcher.pl -i masses.list -g Something.out

-h	displays this usage statement (also using --help)					e.g., perl Antibase_mass_searcher.pl -h	
-i 	input tab-delimited file containing the molecular mass list	REQUIRED		e.g., perl Antibase_mass_searcher.pl -i masses.list
-j 	applied error ranges, in the format \"<starting at this mass-1>,<+/- mass units-1> ... <starting at this mass-n>,<+/- mass units-n>\"
		DEFAULT: \"0,0.002 500,0.004\"				e.g., \"0,0\"

-k	does the input molecular mass list contain a table header?	DEFAULT: Y		e.g., perl Antibase_mass_searcher.pl -i masses.list -k N
-o	output: input file with the Antibase matches appended		
		DEFAULT: Antibase_matches.table				e.g., perl Antibase_mass_searcher.pl -i Antibase.flatfile -o Something.out

-q	run quietly (no printed output)								e.g., perl Antibase_mass_searcher.pl -i Antibase.flatfile -q

OUTPUT FILES:
	Antibase_IDs.table (remaned using -g flag)			Output table of unique masses with their Antibase matches
	Antibase_matches.table (renamed using -o flag)			Outputted version of the input mass table with the Antibase matches appended

Allowable Antibase field names:
	Activity, Appearance, C-NMR, C-NMR-Struct, CA, CAS-RN, ChemClass, Formula, Group, H-NMR, IR-Spectra, Isolation, Mass, MeltingP, MS-Spectra, Name, 
	Opt-Rot, REG, References, Remarks, Solubility, Sources, StructCalc, TLC, Toxicity, UV-A, UV-B, UV-N, UV, All
	\"StructCalc\" and \"Sources\" are required defaults because Antibase_mass_searcher.pl relies on those information 

Allowable Antibase taxonomic descriptions (either words or single letter abbreviations can be used):
	A (Algae), An (Animal), B (Bacteria), C (Cyanobacteria), F (Fungi), L (Lichen), M (Myxobacteria), P (Protist), Pl (Plant), S (Slime mold), 
	Y (Yeast), All (All organisms)

Allowable mass ions to consider:
	M+, M-, MH+, MH-, MNa+
";

# input arguments

%options = ();
GetOptions(
	"a=i" => \$options{mass_column},		"b=i" => \$options{decimal_places},
	"c=s" => \$options{antibase_fields},		"d=s" => \$options{antibase_input_table},
	"e=s" => \$options{antibase_taxa},		"f=s" => \$options{mass_ions},
	"g=s" => \$options{output_IDs},			"h" => \$options{help},
	"help" => \$options{help},			"i=s" => \$options{infile},
	"j=s" => \$options{error_ranges},		"k=s" => \$options{input_file_header},
	"o=s" => \$options{output_matches},		"q" => \$options{quiet}
);

# display usage statement if called

die $usage if ($options{help});

# required arguments

die $usage if (!$options{infile});

# default arguments

$options{mass_column} = 8 if (!$options{mass_column});
$options{decimal_places} = 3 if (!$options{decimal_places});
$options{antibase_fields} = "Name, Sources, Activity, StructCalc" if (!$options{antibase_fields});
$options{antibase_input_table} = "Antibase.table" if (!$options{antibase_input_table});
$options{antibase_taxa} = "All" if (!$options{antibase_taxa});
$options{mass_ions} = "M+, MH+, MNa+" if (!$options{mass_ions});
$options{output_IDs} = "Antibase_IDs.table" if (!$options{output_IDs});
$options{error_ranges} = "0,0.002 500,0.004" if (!$options{error_ranges});
$options{input_file_header} = "Y" if (!$options{input_file_header});
$options{output_matches} = "Antibase_matches.table" if (!$options{output_matches});

# mystery input flags not allowed

die "ERROR: Unrecognized command line arguments: @ARGV\n" if ($ARGV[0]);

# checks that input files exist

unless (-e $options{infile}){ die "ERROR: $options{infile} does not exist\n$usage" }
unless (-e $options{antibase_input_table}){ die "ERROR: $options{antibase_input_table} does not exist\n$usage" }
unless ($options{quiet}){
	unless (-e $options{output_IDs}){ print "WARNING: $options{output_IDs} exists and has been overwritten\n" }
	unless (-e $options{output_matches}){ print "WARNING: $options{output_matches} exists and has been overwritten\n" }
}

# checks for valid Antibase fields; hash keyed to order in input table

%antibase_fields = (
	'CAS-RN' => 1,		Name => 2, 		Formula => 3,		Mass => 4,
	Sources => 5,		Isolation => 6,		'C-NMR' => 7,		'C-NMR-Struct' => 8,
	'H-NMR' => 9,		'MS-Spectra' => 10, 	'UV-A' => 11, 		'UV-B' => 12,
	'UV-N' => 13,		UV => 14, 		'IR-Spectra' => 15,	Toxicity => 16,
	Solubility => 17,	Activity => 18, 	Appearance => 19,	MeltingP => 20, 
	TLC => 21, 		StructCalc => 22,	Group => 23, 		Remarks => 24,
	Refereces => 25,	CA => 26, 		REG => 27,		ChemClass => 28,
	'Opt-Rot' => 29
);

$options{antibase_fields} =~ s/\"//g;
die "ERROR: Antibase fields are formatted incorrectly\n$usage" unless ($options{antibase_fields} =~ /\w+/);
@input_antibase_fields = split ", ", $options{antibase_fields};
foreach (@input_antibase_fields){
	if ($_ eq "All" or $_ eq "all"){
		@input_antibase_fields = keys %antibase_fields;
		last;
	}
	die "ERROR: $_ is not a recognized Antibase field\n$usage" unless ($antibase_fields{$_});
}
unless (grep { /StructCalc/ } @input_antibase_fields) { push @input_antibase_fields, "StructCalc" }
unless (grep { /Sources/ } @input_antibase_fields) { push @input_antibase_fields, "Sources" }

# checks for valid Antibase taxa

%antibase_taxon_abbreviations = (
	A => Algae,		An => Animal,		B => Bacteria,
	C => Cyanobacteria,	F => Fungus,		L => Lichen, 
	M => Myxobacteria,	P => Protist,		Pl => Plant, 
	S => "Slime mold",	Y => Yeast
);

foreach (keys %antibase_taxon_abbreviations){
	$antibase_taxon_names{$antibase_taxon_abbreviations{$_}} = $_;
}

$options{antibase_taxa} =~ s/\"//g;
die "ERROR: Antibase taxa are formatted incorrectly\n$usage" unless ($options{antibase_taxa} =~ /\w+/);
@temp = split ", ", $options{antibase_taxa};
foreach (@temp){
	if ($_ eq "All" or $_ eq "all" or $_ eq "All organisms" or $_ eq "all organisms"){
		@input_antibase_taxa = keys %antibase_taxon_abbreviations;
		last;
	}
	
	if ($antibase_taxon_abbreviations{$_}){ push @input_antibase_taxa, $_ }
	elsif ($antibase_taxon_names{$_}){ push @input_antibase_taxa, $antibase_taxon_names{$_} }
	else { die "ERROR: $_ is not a recognized Antibase field\n$usage" unless ($antibase_taxon_abbreviations{$_} or $antibase_taxon_names) }
}

# checks for valid mass ions

$mass_ions{'M+'} = $mass_ions{'M-'} = $mass_ions{'MH+'} = $mass_ions{'MH-'} = $mass_ions{'MNa+'} = "Y";
%add_to_masses = (
	'M+' => -0.000549,
	'M-' => 0.000549,
	'MH+' => 1.007276,
	'MH-' => 1.008374,
	'MNa+' => 22.98922
);

$options{mass_ions} =~ s/\"//g;
die "ERROR: Antibase taxa are formatted incorrectly\n$usage" unless ($options{mass_ions} =~ /\w+/);
@input_mass_ions = split ", ", $options{mass_ions};
foreach (@input_mass_ions){ die "$_ is not a recognized mass ion\n$usage" unless ($mass_ions{$_}) }

# checks if the number of input decimal places exceeds that for the reference ions

if ($options{decimal_places} > 6){ print "WARNING: Number of input decimal places exceeds maximum; rounding to 6 places\n" }

# checks that error ranges are interpretable, rounds as needed and sets arrays

@input_error_ranges = split " ", $options{error_ranges};
foreach $input (@input_error_ranges){
	die "ERROR: Error ranges are formatted incorrectly\n$usage" unless ($input =~ /((\d|\.)+),((\d|\.)+)/);
	$start = $1;
	$error = $3;
	$decimal_place_corrector = 10 ** $options{decimal_places};
	$error = sprintf ("%.$options{decimal_places}f", int(($error * $decimal_place_corrector) + 0.5) / $decimal_place_corrector);
	$error_range{$start} = $error;
}

# print parameters unless -q flag selected

print "
----------------------------------------------------------------
Antibase_mass_searcher.pl	Jonathan Klassen 	v1.21: October 17, 2012

parameters used:
	-a (column in the input mass list containing the masses, counting from 1) = $options{mass_column}
	-b (number of decimal places included in the mass list) = $options{decimal_places}
	-c (which Antibase fields to display, in the format \"<field-1>, ..., <field-n>\") = @input_antibase_fields
	-d (input Antibase table file) = $options{antibase_input_table}
	-e (which Antibase taxa to display, in the format \"<taxon-1>, ..., <taxon-n>\") = @input_antibase_taxa
	-f (mass ions to consider, in the format \"<ion-1>, ..., <ion-n>\") = @input_mass_ions
	-g (output file of unique masses and their IDs) = $options{output_IDs}
	-i (input tab-delimited file containing the molecular mass list) = $options{infile}
	-j (applied error ranges, in the format \"<starting mass-1>,<+/- mass units-1> ... <starting mass-1n>,<+/- mass units-n>\" = $options{error_ranges}
	-k (does the input molecular mass list contain a table header?) = $options{input_file_header}
	-o (output: input file with the Antibase matches appended) = $options{output_matches}
----------------------------------------------------------------
" unless ($options{quiet}); 

######################################################################################################################
# reads through the antibase table file and stores everything as a hash
# first key is the antibase db id, second is each db field type
######################################################################################################################

open (INANTIBASE, $options{antibase_input_table}) or die "Cannot open $options{antibase_input_table}";

%found_masses = ();
unless ($options{quiet}){ print "Parsing input antibase database...\n" }
$counter = 0;
while (<INANTIBASE>){
	chomp;
	unless ($counter == 0){ # skips table header line
		$counter = 1;
		next;
	}

	# creates hash of the antibase db

	@antibase_line = split "\t";

	# skips db entries without structural information

	next if ($antibase_line[22] eq "no_data"); 
	$antibase_line[22] =~ s/ \.\.\..*$//;

	# skips db entries that do not have organism codes matching those in the inputs

	$next_counter = 0; 
	foreach (@input_antibase_taxa){
		if ($_ eq "all"){
			$next_counter = 1;
		}
		elsif ($antibase_line[5] =~ /(\[$_\])/){
			$next_counter = 1;
		}
	}
	next unless ($next_counter == 1);

	# Determines which fields to include in the output file

	foreach $input_antibase_field (@input_antibase_fields){
		$antibase_hash{$antibase_line[0]}{$input_antibase_field} = $antibase_line[$antibase_fields{$input_antibase_field}];
	}

	# creates a hash indexing an array of ids by their mass

		# rounding logic: 
		# (i) multiply by 10^decimal_places to bring the decimal place to be rounded adjacent to the decimal point
		# (ii) add 0.5 so if above the rounding threshold the digit to the left of the decimal is incremented
		# (iii) int reduces to the previous integer
		# (iv) divide by 10^decimal_places to get the decimals back

	$decimal_place_corrector = 10 ** $options{decimal_places};

	foreach $input_mass_ion (@input_mass_ions){
		$temp = $antibase_line[22] + $add_to_masses{$input_mass_ion};
		$a = sprintf ("%.$options{decimal_places}f", int(($temp * $decimal_place_corrector) + 0.5) / $decimal_place_corrector);
		push @{$antibase_ions{$input_mass_ion}{$a}}, $antibase_line[0];
		$found_masses{$a} = "y";
	}
}

######################################################################################################################
# reads through input file containing the masses to be queried
# outputs the match table at the same time
######################################################################################################################

open (INMASS, $options{infile}) or die "Cannot open $options{infile}";
open (OUTMATCH, ">$options{output_matches}") or die "Cannot open $options{output_matches}";

print "Parsing input mass table...\n";

$header_counter = 0;
%found_masses = ();
while (<INMASS>){
	chomp;
	@mass_line = split "\t";
	
	# outputs input table header

	if ($header_counter == 0){
		if ($options{input_file_header} eq "Y" or $options{input_file_header} eq "y"){
			print OUTMATCH join "\t", @mass_line, "\t";
		}
		else {
			foreach (@mass_line){ print OUTMATCH "\t" }
		}
		print OUTMATCH "Antibase_matches key: hit_counter|ion_type"; 
		foreach (@input_antibase_fields){ print OUTMATCH "|", $_ }
		print OUTMATCH "\n";
		$header_counter = 1;
		next;
	}

	# lines are skipped unless a mass number can be recognized, e.g., to allow for column headers

	next unless ($mass_line[$options{mass_column}-1] =~ /((\d|.)+)/);
	$mass = $1;

	# mass is rounded to the appropriate number of decimal places, <= 6

	if ($options{decimal_places} > 6){
		$mass = sprintf ("%.6f", int(($mass * 1000000) + 0.5) / 1000000);
	}
	else {
		$decimal_place_corrector = 10 ** $options{decimal_places};
		$mass = sprintf ("%.$options{decimal_places}f", int(($mass * $decimal_place_corrector) + 0.5) / $decimal_place_corrector);
	}

	# enumerates unique masses in the input file, stores them as an array
	# redundancy possible, e.g., from the same mass at different retention times

	unless ($found_masses{$mass}){
		$found_masses{$mass} = "y";
		push @masses, $mass;
	}

	# outputs input line with masses rounded appropriately

	for (0..$#mass_line){
		if ($_ == $options{mass_column} - 1){
			print OUTMATCH "$mass\t";
		}
		else {
			print OUTMATCH "$mass_line[$_]\t";
		}
	}

	# determines tolerance for this mass

	$tolerance = '';
	foreach (sort {$a <=> $b} keys %error_range){
		if ($mass > $_){
			$tolerance = $error_range{$_};
		}
	}
	die unless ($tolerance);

	# finds matching Antibase matches

	%temp_found = ();
	for ($a = $mass - $tolerance; $a <= $mass + $tolerance; $a += (1 / 10 ** $options{decimal_places})){
		$a = sprintf("%.$options{decimal_places}f", $a);
		foreach $input_mass_ion (@input_mass_ions){
			if ($antibase_ions{$input_mass_ion}{$a}){
				foreach $id (sort @{$antibase_ions{$input_mass_ion}{$a}}){
					push @{$temp_found{$input_mass_ion}}, $id;
				}	
			}
		}
	}

	# outputs found matches, ordered by mass ion type and molecular mass

	$counter = 0;
	foreach $input_mass_ion (@input_mass_ions){
		foreach $id (@{$temp_found{$input_mass_ion}}){
			$counter++;
			print OUTMATCH "[hit_$counter]|$input_mass_ion";
			foreach $field (@input_antibase_fields){
				print OUTMATCH "|$antibase_hash{$id}{$field}";
			}
			print OUTMATCH " -- ";
		}
	}
	print OUTMATCH "\n";
}

######################################################################################################################
# constructs the matching IDs output file
# all matches for each unique mass are enumerated, separated by mass ion
######################################################################################################################

unless ($options{quiet}){ print "Writing matching IDs output file...\n" }

open (OUTFILE, ">$options{output_IDs}") or die "Cannot open $options{output_IDs}";

# outfile header

print OUTFILE "Measured_mass\tMass_ion";
foreach $field (@input_antibase_fields){
	print OUTFILE "\t$field";
}
print OUTFILE "\n";

# for each mass, output matching Antibase hits for each tested ion

@masses = sort {$a <=> $b} @masses;
%found = ();
foreach $mass (@masses){

	# finds matching Antibase matches

	%temp_found = ();
	for ($a = $mass - $tolerance; $a <= $mass + $tolerance; $a += (1 / 10 ** $options{decimal_places})){
		$a = sprintf("%.$options{decimal_places}f", $a);
		foreach $input_mass_ion (@input_mass_ions){
			if ($antibase_ions{$input_mass_ion}{$a}){
				foreach $id (sort @{$antibase_ions{$input_mass_ion}{$a}}){
					push @{$temp_found{$input_mass_ion}}, $id;
				}	
			}
		}
	}

	# outputs found matches, ordered by mass ion type and molecular mass

	$counter = 0;
	foreach $input_mass_ion (@input_mass_ions){
		foreach $id (@{$temp_found{$input_mass_ion}}){
			print OUTFILE "$mass\t$input_mass_ion";
			foreach $field (@input_antibase_fields){
				print OUTFILE "\t$antibase_hash{$id}{$field}";
			}
			print OUTFILE "\n";
			$found{$a} = "y"
		}
	}
	unless ($found{$mass}){
		print OUTFILE "$mass\tno_Antibase_match\n";
	}
}
