#!usr/bin/perl
#
# Antibase_parser_v1.pl
#
# Jonathan Klassen
#
# v1.0: August 29, 2011
#
# Parses an antibase flatfile into one that is tab delimited

use warnings;
use Getopt::Long;

######################################################################################################################
# Processes input arguments
######################################################################################################################

$usage = "
Antibase_parser.pl: v1.0

USAGE:
-h	displays this usage statement (also using --help)				e.g., perl Antibase_parser.pl -h	
-i 	input Antibase flat file 			REQUIRED			e.g., perl Antibase_parser.pl -i Antibase.flatfile
-o	output Antibase table file name			DEFAULT: Antibase.table		e.g., perl Antibase_parser.pl -i Antibase.flatfile -o Something.out
-q	run quietly (no printed output)							e.g., perl Antibase_parser.pl -i Antibase.flatfile -q

OUTPUT FILES:
	Antibase.table (renamed using -o flag)			Antibase reformatted as a more machine-readable tab-delimited table

See the package README for recommendations for making the Antibase flat file	
";

# input arguments

%options = ();
GetOptions(
	"h" => \$options{help},
	"help" => \$options{help},
	"i=s" => \$options{infile},
	"o=s" => \$options{outfile},
	"q" => \$options{quiet}
);

# display usage statement if called

die $usage if ($options{help});

# required arguments

die $usage if (!$options{infile});

# default arguments

$options{outfile} = "Antibase.table" if (!$options{outfile});

# mystery input flags not allowed

die "Unrecognized command line arguments: @ARGV\n" if ($ARGV[0]);

# checks if input and output files exist

unless (-e $options{infile}){ die "$options{infile} does not exist\n$usage" }
unless ($options{quiet}){
	unless (-e $options{outfile}){ print "WARNING: $options{outfile} exists and has been overwritten" }
}

# print parameters unless -q flag selected

print "
----------------------------------------------------------------
Antibase_parser.pl	Jonathan Klassen 	v1.0: August 29, 2012

parameters used:
	-i (Input Antibase flat file) = $options{infile}
	-o (Output Antibase table file name) = $options{outfile}
----------------------------------------------------------------
" unless ($options{quiet}); 

######################################################################################################################
# Reads through input flat file; stores in data hash
######################################################################################################################

@fields = ("CAS-RN", "Name", "Formula", "Mass", "Sources", "Isolation", "C-NMR", "C-NMR-Struct", "H-NMR", "MS-Spectra", "UV-A", "UV-B", "UV-N", "UV", "IR-Spectra",
	"Toxicity", "Solubility", "Activity", "Appearance", "MeltingP", "TLC", "StructCalc", "Group", "Remarks", "References", "CA", "REG", "ChemClass", "Opt-Rot");
for $a (0..$#fields){ # indexes array positions by their name
	$field_counter{$fields[$a]} = $a
}

$counter = 0;
open (INFILE, $options{infile}) or die "Cannot open $options{infile}";
while (<INFILE>){
	s/\s*$//g;	
	s/\//_/g;
	next if (/Compound identification data:/ or /Properties:/ or /AntiBase/); # skips empty and header lines

	# output hash keyed by Comp. ID (integer); each field type is detected and then added at the end (allows for addition of blank lines too during this step)

	if (/Comp. ID: \[(\d )*(.+)\]/){ # output hash keyed by Comp. ID (integer)
		$counter++;
		unless ($options{quiet}){
			if ($counter % 1000 == 0){
				print "processing record $counter...\n";
			}
		}
		$id = $2;
		die unless ($id);
		next;
	}
	elsif (/CAS-RN: (.+)/){
		$found = $1;
		$field_found = "CAS-RN";
	}
	elsif (/Formula: (.+)/){
		$found = $1;
		$field_found = "Formula";
	}
	elsif (/Mass: (.+)/){
		$found = $1;
		$field_found = "Mass";
	}
	elsif (/Name: (.+)/){ 
		$found = $1;
		$field_found = "Name";
	}
	elsif (/Sources (.+)/){
		$found = $1;
		$field_found = "Sources";

		# mistakes in the source field

		$found =~ s/\[A A\]/\[A\]/g;
		$found =~ s/\[B B\]/\[B\]/g;
		$found =~ s/\[C C\]/\[C\]/g;
		$found =~ s/\[F F\]/\[F\]/g;
		$found =~ s/\[M M\]/\[M\]/g;
		$found =~ s/\[P Pl\]/\[P\] \[Pl\]/g;
		$found =~ s/\[Y Y\]/\[Y\]/g;
		$found =~ s/\[a artifact\?\]/\[A\] artifact\?/g;
		unless ($found =~ /\[marine zoo- and phytoplankton\]/){ $found =~ s/\[arine zoo- and phytoplankton\]/\[marine zoo- and phytoplankton\]/g }
		unless ($found =~ /\[C\] Microcystis viridis/){ $found =~ s/\[C9 Microcystis viridis/\[C\] Microcystis viridis/g }
		unless ($found =~ /\[B\] nonspore-forming filamentous microorganism PF1022/){ $found =~ 
			s/\[nonspore-forming filamentous microorganism PF1022\]/\[B\] nonspore-forming filamentous microorganism PF1022/g }
		unless ($found =~ /\[An\] \[Sponge\] Negombata magnifica/){ $found =~ s/\[Sponge\] Negombata magnifica/\[An\] \[Sponge\] Negombata magnifica/g }
		unless ($found =~ /\[B\] A\. fulvoviolaceus var\. achromogenes lia-0721/){ $found =~ 
			s/A\. fulvoviolaceus var\. achromogenes lia-0721/\[B\] A\. fulvoviolaceus var\. achromogenes lia-0721/ }
		unless ($found =~ /\[B\] A\. memokrassinus/){ $found =~ s/A\. memokrassinus/\[B\] A\. memokrassinus/g }
		unless ($found =~ /\[B\] A\. rectilavendulae var\. pentaenicus lia-0777/){ $found =~ 
			s/A\. rectilavendulae var\. pentaenicus lia-0777/\[B\] A\. rectilavendulae var\. pentaenicus lia-0777/g }
		unless ($found =~ /\[A\] red alga Laurencia sp\./){ $found =~ s/A\] red alga Laurencia sp\./\[A\] red alga Laurencia sp\./g }
		unless ($found =~ /\[F\] Acrostalagmus sp\., fungus Oidiodendron truncatum, O\. cf\. truncatum, O\. griseum/){ $found =~ 
			s/Acrostalagmus sp\., fungus Oidiodendron truncatum, O\. cf\. truncatum, O\. griseum/\[F\] Acrostalagmus sp\., fungus Oidiodendron truncatum, O\. cf\. truncatum, O\. griseum/g }
		unless ($found =~ /\[B\] Actinomyces mh193-16f4 \(FERM bp-2051\)/){ $found =~ 
			s/Actinomyces mh193-16f4 \(FERM bp-2051\)/\[B\] Actinomyces mh193-16f4 \(FERM bp-2051\)/g }
		unless ($found =~ /\[F\] Agonomyces sp\. f-11605/){ $found =~ s/Agonomyces sp\. f-11605/\[F\] Agonomyces sp\. f-11605/g }
		unless ($found =~ /\[F\] Agonomyces sp\. pf 1022 \(mycelia sterilia\)/){ $found =~ 
			s/Agonomyces sp\. pf 1022 \(mycelia sterilia\)/\[F\] Agonomyces sp\. pf 1022 \(mycelia sterilia\)/g }
		unless ($found =~ /\[B\] Alginovibrio verticillis/){ $found =~ s/Alginovibrio verticillis/\[B\] Alginovibrio verticillis/g }
		unless ($found =~ /\[B\] Amycolatopsis fastidiosa/){ $found =~ s/Amycolatopsis fastidiosa/\[B\] Amycolatopsis fastidiosa/g }
		unless ($found =~ /\[B\] Amycolatopsis orientalis ncib 40011/){ $found =~ 
			s/Amycolatopsis orientalis ncib 40011/\[B\] Amycolatopsis orientalis ncib 40011/g }
		unless ($found =~ /\[B\] Amycolatopsis strain 17128/){ $found =~ s/Amycolatopsis strain 17128/\[B\] Amycolatopsis strain 17128/g }
		unless ($found =~ /\[F\] Antrodia camphorataa/){ $found =~ s/Antrodia camphorata/\[F\] Antrodia camphorataa/g }
		unless ($found =~ /\[Pl\] Anvillea garcini and Zoegea baldschuanica/){ 
			$found =~ s/Anvillea garcini and Zoegea baldschuanica/\[Pl\] Anvillea garcini and Zoegea baldschuanica/g }
		unless ($found =~ /\[S\] Arcyria cinerea/){ $found =~ s/Arcyria cinerea/\[S\] Arcyria cinerea/g }
		unless ($found =~ /\[B\] AS-1387392 \+ actinomycetes/){ $found =~ s/AS-1387392 \+ actinomycetes/\[B\] AS-1387392 \+ actinomycetes/g }
		unless ($found =~ /\[S\] Arcyria cinerea/){ $found =~ s/Arcyria cinerea/\[S\] Arcyria cinerea/g }
		unless ($found =~ /\[F\] Ascotricha amphitricha/){ $found =~ s/Ascotricha amphitricha/\[F\] Ascotricha amphitricha/g }
		unless ($found =~ /\[F\] Aspergillus alliaceus \+ 5-Methoxyflavone/){ $found =~ 
			s/Aspergillus alliaceus \+ 5-Methoxyflavone/\[F\] Aspergillus alliaceus \+ 5-Methoxyflavone/g }
		unless ($found =~ /\[F\] Aspergillus flavus \+ 7,8-dimethoxyflavoneroduced/){ $found =~ 
			s/Aspergillus flavus \+ 7,8-dimethoxyflavoneroduced/\[F\] Aspergillus flavus \+ 7,8-dimethoxyflavoneroduced/g }
		unless ($found =~ /\[Pl\] Asteriscus vogelii/){ $found =~ s/Asteriscus vogelii/\[Pl\] Asteriscus vogelii/g }
		unless ($found =~ /\[F\] ATCC 74256/){ $found =~ s/ATCC 74256/\[F\] ATCC 74256/g }
		unless ($found =~ /\[B\] Bacillus amyloliquefaciens LP03/){ $found =~ s/Bacillus amyloliquefaciens LP03/\[B\] Bacillus amyloliquefaciens LP03/g }
		unless ($found =~ /\[B\] Bacillus cereus/){ $found =~ s/Bacillus cereus/\[B\] Bacillus cereus/g }
		unless ($found =~ /\[B\] Bacillus subtilis KSM 6-10/){ $found =~ s/Bacillus subtilis KSM 6-10/\[B\] Bacillus subtilis KSM 6-10/g }
		unless ($found =~ /\[F\] basidiomycete Lactarius hatsudake \(Russulaceae\)/){ $found =~ 
			s/basidiomycete Lactarius hatsudake \(Russulaceae\)/\[F\] basidiomycete Lactarius hatsudake \(Russulaceae\)/g }
		unless ($found =~ /\[F\] Blastocladia ramosa,\[F\] Nectria galligena,\[An\] Leucopaxillus giganteus, \[An\] sponge Axinella cannabina/){ $found =~ 
			s/Blastocladia ramosa, Nectria galligena, \[An\] Leucopaxillus giganteus, sponge Axinella cannabina/\[F\] Blastocladia ramosa,\[F\] Nectria galligena,\[An\] Leucopaxillus giganteus, \[An\] sponge Axinella cannabina/g }
		unless ($found =~ /\[F\] Boletus pulverulentus Opat/){ $found =~ s/Boletus pulverulentus Opat/\[F\] Boletus pulverulentus Opat/g }
		unless ($found =~ /\[A\] brown alga Desmarestia menziesii/){ $found =~ s/brown alga Desmarestia menziesii/\[A\] brown alga Desmarestia menziesii/g }
		unless ($found =~ /\[A\] bbrown algae, \[An\] sponges/){ $found =~ s/brown algae, \[An\] sponges/\[A\] bbrown algae, \[An\] sponges/g }
		unless ($found =~ /\[Pl\] Brucea amarissima/){ $found =~ s/Brucea amarissima/\[Pl\] Brucea amarissima/g }
		unless ($found =~ /\[Pl\] Brucea javanica/){ $found =~ s/Brucea javanica/\[Pl\] Brucea javanica/g }
		unless ($found =~ /\[B\] Burkholderia cepacia/){ $found =~ s/Burkholderia cepacia/\[B\] Burkholderia cepacia/g }
		unless ($found =~ /\[L\] C\. arbuscula/){ $found =~ s/C\. arbuscula/\[L\] C\. arbuscula/g }
		unless ($found =~ /\[F\] C\. cladosporioides kac-2215/){ $found =~ s/C\. cladosporioides kac-2215/\[F\] C\. cladosporioides kac-2215/g }
		unless ($found =~ /\[B\] C\. hydrocarboclastus/){ $found =~ s/C\. hydrocarboclastus/\[B\] C\. hydrocarboclastus/g }
		unless ($found =~ /\[F\] C\. lunata/){ $found =~ s/C\. lunata/\[F\] C\. lunata/g }
		unless ($found =~ /\[F\] C\. striatus/){ $found =~ s/C\. striatus/\[F\] C\. striatus/g }
		unless ($found =~ /\[F\] C\. virescens var\. thielavioideum/){ $found =~ 
			s/C\. virescens var\. thielavioideum/\[F\] C\. virescens var\. thielavioideum/g }
		unless ($found =~ /\[An\] Cacospongia scalaris/){ $found =~ s/Cacospongia scalaris/\[An\] Cacospongia scalaris/g }
		unless ($found =~ /\[A\] chrysophyte Ochromonas danica IAM CS-2/){ $found =~ 
			s/chrysophyte Ochromonas danica IAM CS-2/\[A\] chrysophyte Ochromonas danica IAM CS-2/g }
		unless ($found =~ /\[Pl\] Clerodendron brachyanthum\; Stellera chamaejasme/){ $found =~ 
			s/Clerodendron brachyanthum\; Stellera chamaejasme/\[Pl\] Clerodendron brachyanthum\; Stellera chamaejasme/g }
		unless ($found =~ /\[An\] cockle Austrovenus stutchburyi/){ $found =~ s/cockle Austrovenus stutchburyi/\[An\] cockle Austrovenus stutchburyi/g }
		unless ($found =~ /\[F\] Coelomycetes sp\. FO-2546/){ $found =~ s/Coelomycetes sp\. FO-2546/\[F\] Coelomycetes sp\. FO-2546/g }
		unless ($found =~ /\[M\] Corallococcus coralloides/){ $found =~ s/Corallococcus coralloides/\[M\] Corallococcus coralloides/g }
		unless ($found =~ /\[F\] Cronartium fusiforme,\[F\] Melampsora lini \(FM 263\), [An] sponge Axinella cannabina/){ $found =~ 
			s/Cronartium fusiforme, Melampsora lini \(FM 263\), [An] sponge Axinella cannabina/\[F\] Cronartium fusiforme,\[F\] Melampsora lini \(FM 263\), [An] sponge Axinella cannabina/g }
		unless ($found =~ /\[Pl\] Crotolaria sp\./){ $found =~ s/Crotolaria sp\./\[Pl\] Crotolaria sp\./g }
		unless ($found =~ /\[F\] cultured mycobiont from [L] lichen Haematomma sp./){ $found =~ 
			s/cultured mycobiont from [L] lichen Haematomma sp./\[F\] cultured mycobiont from [L] lichen Haematomma sp./g }
		unless ($found =~ /\[Pl\] Cyathostemma viridiflorum griff/){ $found =~ s/Cyathostemma viridiflorum griff/\[Pl\] Cyathostemma viridiflorum griff/g }
		unless ($found =~ /\[An\] deep sea crinoid Gymnochrinus richeri/){ $found =~ 
			s/deep sea crinoid Gymnochrinus richeri/\[An\] deep sea crinoid Gymnochrinus richeri/g }
		unless ($found =~ /\[D\] Diacronema sp\./){ $found =~ s/Diacronema sp\./\[D\] Diacronema sp\./g }
		unless ($found =~ /\[Pl\] Discaria febrifuga martius/){ $found =~ s/Discaria febrifuga martius/\[Pl\] Discaria febrifuga martius/g }
		unless ($found =~ /\[F\] Discomyceta sp\./){ $found =~ s/Discomyceta sp\./\[F\] Discomyceta sp\./g }
		unless ($found =~ /\[Pl\] Eichhornia crassipes/){ $found =~ s/Eichhornia crassipes/\[Pl\] Eichhornia crassipes/g }
		unless ($found =~ /\[F\] Emericella purpurea/){ $found =~ s/Emericella purpurea/\[F\] Emericella purpurea/g }
		unless ($found =~ /\[F\] Engleromyces goetzei/){ $found =~ s/Engleromyces goetzei/\[F\] Engleromyces goetzei/g }
		unless ($found =~ /\[An\] Euplexaura sp\./){ $found =~ s/Euplexaura sp\./\[An\] Euplexaura sp\./g }
		unless ($found =~ /\[An\] Fascaplysinopsis sp\./){ $found =~ s/Fascaplysinopsis sp\./\[An\] Fascaplysinopsis sp\./g }
		unless ($found =~ /\[An\] Ficulina ficus/){ $found =~ s/Ficulina ficus/\[An\] Ficulina ficus/g }
		unless ($found =~ /\[F\] Flavolaschia calocera/){ $found =~ s/Flavolaschia calocera/\[F\] Flavolaschia calocera/g }
		unless ($found =~ /\[F\] fungus Myrothecium verrucaria/){ $found =~ s/fungus Myrothecium verrucaria/\[F\] fungus Myrothecium verrucaria/g }
		unless ($found =~ /\[F\] G\. humicola/){ $found =~ s/G\. humicola/\[F\] G\. humicola/g }
		unless ($found =~ /\[Pl\] G\. subcordata/){ $found =~ s/G\. subcordata/\[Pl\] G\. subcordata/g }
		unless ($found =~ /\[F\] Ganoderma lucidum/){ $found =~ s/Ganoderma lucidum/\[F\] Ganoderma lucidum/g }
		unless ($found =~ /\[Pl\] Geijera parviflora/){ $found =~ s/Geijera parviflora/\[Pl\] Geijera parviflora/g }
		unless ($found =~ /\[Pl\] Gnidia glaucus/){ $found =~ s/Gnidia glaucus/\[Pl\] Gnidia glaucus/g }
		unless ($found =~ /\[Pl\] Gnidia latifolia/){ $found =~ s/Gnidia latifolia/\[Pl\] Gnidia latifolia/g }
		unless ($found =~ /\[Pl\] Gnidia subcordata/){ $found =~ s/Gnidia subcordata/\[Pl\] Gnidia subcordata/g }
		unless ($found =~ /\[An\] Gonyleptideae sp\./){ $found =~ s/Gonyleptideae sp\./\[An\] Gonyleptideae sp\./g }
		unless ($found =~ /\[Pl\] Hakea trifurcata/){ $found =~ s/Hakea trifurcata/\[Pl\] Hakea trifurcata/g }
		unless ($found =~ /\[Pl\] Helicteras angustifolia/){ $found =~ s/Helicteras angustifolia/\[Pl\] Helicteras angustifolia/g }
		unless ($found =~ /\[F\] Holwaya mutida/){ $found =~ s/Holwaya mutida/\[F\] Holwaya mutida/g }
		unless ($found =~ /\[F\] Hyphozyma lignicola/){ $found =~ s/Hyphozyma lignicola/\[F\] Hyphozyma lignicola/g }
		unless ($found =~ /\[An\] Ianthella ardis/){ $found =~ s/Ianthella ardis/\[An\] Ianthella ardis/g }
		unless ($found =~ /\[Pl\] Kielmeyera coriacea/){ $found =~ s/Kielmeyera coriacea/\[Pl\] Kielmeyera coriacea/g }
		unless ($found =~ /\[B\] Kocuria sp\./){ $found =~ s/Kocuria sp\./\[B\] Kocuria sp\./g }
		unless ($found =~ /\[Pl\] Koelpinia linearis/){ $found =~ s/Koelpinia linearis/\[Pl\] Koelpinia linearis/g }
		unless ($found =~ /\[F\] Lactarius hatsudake/){ $found =~ s/Lactarius hatsudake/\[F\] Lactarius hatsudake/g }
		unless ($found =~ /\[B\] Lactobacillus plantarum LPCO10/){ $found =~ s/Lactobacillus plantarum LPCO10/\[B\] Lactobacillus plantarum LPCO10/g }
		unless ($found =~ /\[B\] Lactobacillus plantarum TF711/){ $found =~ s/Lactobacillus plantarum TF711/\[B\] Lactobacillus plantarum TF711/g }
		unless ($found =~ /\[A\] Laurencia microcladia/){ $found =~ s/Laurencia microcladia/\[A\] Laurencia microcladia/g }
		unless ($found =~ /\[An\] Luffariella variabilis/){ $found =~ s/Luffariella variabilis/\[An\] Luffariella variabilis/g }
		unless ($found =~ /\[Pl\] M\. deserti/){ $found =~ s/M\. deserti/\[Pl\] M\. deserti/g }
		unless ($found =~ /\[F\] M\. phaseolina/){ $found =~ s/M\. phaseolina/\[F\] M\. phaseolina/g }
		unless ($found =~ /\[F\] M\. verrucaria/){ $found =~ s/M\. verrucaria/\[F\] M\. verrucaria/g }
		unless ($found =~ /\[Pl\] Maesa macrophylla/){ $found =~ s/Maesa macrophylla/\[Pl\] Maesa macrophylla/g }
		unless ($found =~ /\[F\] mangrove endophytic fungus \#2240/){ $found =~ 
			s/mangrove endophytic fungus \#2240/\[F\] mangrove endophytic fungus \#2240/g }
		unless ($found =~ /\[F\] mangrove endophytic fungus no 2240/){ $found =~ 
			s/mangrove endophytic fungus no 2240/\[F\] mangrove endophytic fungus no 2240/g }
		unless ($found =~ /\[Pl\] Maprounea africana, Homonoia riparia/){ $found =~ 
			s/Maprounea africana, Homonoia riparia/\[Pl\] Maprounea africana, Homonoia riparia/g }
		unless ($found =~ /\[P\] marine ciliate Maristentor dinoferus/){ $found =~ 
			s/marine ciliate Maristentor dinoferus/\[P\] marine ciliate Maristentor dinoferus/g }
		unless ($found =~ /\[An\] marine invertebrate Lemnalia africana; \[B\] Streptomyces sp\. GT 061115, Streptomyces sp\. HKI 0297/){ $found =~ 
			s/marine invertebrate Lemnalia africana; \[B\] Streptomyces sp\. GT 061115, Streptomyces sp\. HKI 0297/\[An\] marine invertebrate Lemnalia africana; \[B\] Streptomyces sp\. GT 061115, Streptomyces sp\. HKI 0297/g }
		unless ($found =~ /\[B\] Marine Pathogenic Bacterium Vibrio harveyi/){ $found =~ 
			s/Marine Pathogenic Bacterium Vibrio harveyi/\[B\] Marine Pathogenic Bacterium Vibrio harveyi/g }
		unless ($found =~ /\[An\] marine sponge/){ $found =~ s/marine sponge/\[An\] marine sponge/g }
		unless ($found =~ /\[B\] marine Streptomyces sp\. NTK 227/){ $found =~ s/marine Streptomyces sp\. NTK 227/\[B\] marine Streptomyces sp\. NTK 227/g }
		unless ($found =~ /\[F\] MF5453 \(ATCC 20986\)/){ $found =~ s/MF5453 \(ATCC 20986\)/\[F\] MF5453 \(ATCC 20986\)/g }
		unless ($found =~ /\[B\] Micromonospora echinospora ssp\. challisensis NRRL 12255/){ $found =~ 
			s/Micromonospora echinospora ssp\. challisensis NRRL 12255/\[B\] Micromonospora echinospora ssp\. challisensis NRRL 12255/g }
		unless ($found =~ /\[B\] Micropolyspora sp.\ FERM BP-538\; Faenia interjecta sp\. nov\./){ $found =~ 
			s/Micropolyspora sp\. FERM BP-538\; Faenia interjecta sp\. nov\./\[B\] Micropolyspora sp.\ FERM BP-538\; Faenia interjecta sp\. nov\./g }
		unless ($found =~ /\[An\] mollusk Elysia rufescens\; \[A\] green alga Bryopsis sp\. \[B\] \[A\] Vibrio sp\. associated with Bryopsis sp\.and Elysia rufescens/){ $found =~ 
			s/mollusk Elysia rufescens\; \[A\] green alga Bryopsis sp\. \[B\] \[A\] Vibrio sp\. associated with Bryopsis sp\.and Elysia rufescens/\[An\] mollusk Elysia rufescens\; \[A\] green alga Bryopsis sp\. \[B\] \[A\] Vibrio sp\. associated with Bryopsis sp\.and Elysia rufescens/g }
		unless ($found =~ /\[An\] mollusk Elysia rufescens\; green alga \[A\] Bryopsis sp\./){ $found =~ 
			s/mollusk Elysia rufescens\; green alga \[A\] Bryopsis sp\./\[An\] mollusk Elysia rufescens\; green alga \[A\] Bryopsis sp\./g }
		unless ($found =~ /\[F\] Mucor ramannianus \+ 7,8-dimethoxyflavone/){ $found =~ 
			s/Mucor ramannianus \+ 7,8-dimethoxyflavone/\[F\] Mucor ramannianus \+ 7,8-dimethoxyflavone/g }
		unless ($found =~ /\[B\] Mycolatopsis trehalostatica/){ $found =~ s/Mycolatopsis trehalostatica/\[B\] Mycolatopsis trehalostatica/g }
		unless ($found =~ /\[B\] N\. sp\. C-14482 \(ATCC 31309, FERM-p 4130, IFO 13725\), and its mutant \(n-1020\) \(ATCC 31487,FERM-p 4779, IFO 13887\)/){ $found =~ 
			s/N\. sp\. C-14482 \(ATCC 31309, FERM-p 4130, IFO 13725\), and its mutant \(n-1020\) \(ATCC 31487,FERM-p 4779, IFO 13887\)/\[B\] N\. sp\. C-14482 \(ATCC 31309, FERM-p 4130, IFO 13725\), and its mutant \(n-1020\) \(ATCC 31487,FERM-p 4779, IFO 13887\)/g }
		unless ($found =~ /\[Pl\] N\. sulphurea/){ $found =~ s/N\. sulphurea/\[Pl\] N\. sulphurea/g }
		unless ($found =~ /\[B\] Nocardia nova JCM 6044/){ $found =~ s/Nocardia nova JCM 6044/\[B\] Nocardia nova JCM 6044/g }
		unless ($found =~ /\[B\] Nonomuraea turkmeniaca MA7364/){ $found =~ s/Nonomuraea turkmeniaca MA7364/\[B\] Nonomuraea turkmeniaca MA7364/g }
		unless ($found =~ /\[F\] Pecicula sp\./){ $found =~ s/Pecicula sp\./\[F\] Pecicula sp\./g }
		unless ($found =~ /\[F\] Penicillium rubrum/){ $found =~ s/Penicillium rubrum/\[F\] Penicillium rubrum/g }
		unless ($found =~ /\[F\] Pereniporia medullaepanis/){ $found =~ s/Pereniporia medullaepanis/\[F\] Pereniporia medullaepanis/g }
		unless ($found =~ /\[B\] Pfeiferella whitmori/){ $found =~ s/Pfeiferella whitmori/\[B\] Pfeiferella whitmori/g }
		unless ($found =~ /\[A\] Phacelocarpus labillardieri/){ $found =~ s/Phacelocarpus labillardieri/\[A\] Phacelocarpus labillardieri/g }
		unless ($found =~ /\[F\] plant endophytic Pestalotiopsis foedans/){ $found =~ 
			s/plant endophytic Pestalotiopsis foedans/\[F\] plant endophytic Pestalotiopsis foedans/g }
		unless ($found =~ /\[D\] prasinophytes/){ $found =~ s/prasinophytes/\[D\] prasinophytes/g }
		unless ($found =~ /\[Pl\] Pseudolarix kaempferi/){ $found =~ s/Pseudolarix kaempferi/\[Pl\] Pseudolarix kaempferi/g }
		unless ($found =~ /\[B\] Pseudomonas syringae pv\. lachrymans strain 508/){ $found =~ 
			s/Pseudomonas syringae pv\. lachrymans strain 508/\[B\] Pseudomonas syringae pv\. lachrymans strain 508/g }
		unless ($found =~ /\[An\] puffer fish Arothron nigropunctatus, [B] Fugu niphobles, marine Vibrio sp., Alteromonas sp./){ $found =~ 
			s/puffer fish Arothron nigropunctatus, [B] Fugu niphobles, marine Vibrio sp., Alteromonas sp./\[An\] puffer fish Arothron nigropunctatus, [B] Fugu niphobles, marine Vibrio sp., Alteromonas sp./g }
		unless ($found =~ /\[F\] Ramularia collo-cygni/){ $found =~ s/Ramularia collo-cygni/\[F\] Ramularia collo-cygni/g }
		unless ($found =~ /\[A\] red alga Gracillaria verrucosa/){ $found =~ s/red alga Gracillaria verrucosa/\[A\] red alga Gracillaria verrucosa/g }
		unless ($found =~ /\[B\] S\. hygroscopicus k-41 \(FERM-p 1342\), S\. albus \(NRRL 11109\)/){ $found =~ 
			s/S\. hygroscopicus k-41 \(FERM-p 1342\), S\. albus \(NRRL 11109\)/\[B\] S\. hygroscopicus k-41 \(FERM-p 1342\), S\. albus \(NRRL 11109\)/g }
		unless ($found =~ /\[B\] S\.hygroscopicus-azalomyceticus, Act\.sp\., Act\.imbricatus/){ $found =~ 
			s/S\.hygroscopicus-azalomyceticus, Act\.sp\., Act\.imbricatus/\[B\] S\.hygroscopicus-azalomyceticus, Act\.sp\., Act\.imbricatus/g }
		unless ($found =~ /\[B\] Saccharothrix sp\. AJ9571/){ $found =~ s/Saccharothrix sp\. AJ9571/\[B\] Saccharothrix sp\. AJ9571/g }
		unless ($found =~ /\[F\] Samarospora sp\./){ $found =~ s/Samarospora sp\./\[F\] Samarospora sp\./g }
		unless ($found =~ /\[Pl\] Schizandra nigra/){ $found =~ s/Schizandra nigra/\[Pl\] Schizandra nigra/g }
		unless ($found =~ /\[An\] soft coral Sarcophyton crassocaule/){ $found =~ 
			s/soft coral Sarcophyton crassocaule/\[An\] soft coral Sarcophyton crassocaule/g }
		unless ($found =~ /\[Pl\] Spathelia glabrescens/){ $found =~ s/Spathelia glabrescens/\[Pl\] Spathelia glabrescens/g }
		unless ($found =~ /\[F\] Sphaeropsidales sp\. F-24707/){ $found =~ s/Sphaeropsidales sp\. F-24707/\[F\] Sphaeropsidales sp\. F-24707/g }
		unless ($found =~ /\[F\] Spicaria elegans \+ cytochrome P 450 inhibitor/){ $found =~ 
			s/Spicaria elegans \+ cytochrome P 450 inhibitor/\[F\] Spicaria elegans \+ cytochrome P 450 inhibitor/g }
		unless ($found =~ /\[B\] St\. hygroscopicus var\. hialomyceticus 4236/){ $found =~ 
			s/St\. hygroscopicus var\. hialomyceticus 4236/\[B\] St\. hygroscopicus var\. hialomyceticus 4236/g }
		unless ($found =~ /\[B\] Staphylococcus warneri RB4/){ $found =~ s/Staphylococcus warneri RB4/\[B\] Staphylococcus warneri RB4/g }
		unless ($found =~ /\[Pl\] Stellera chamaejasme/){ $found =~ s/Stellera chamaejasme/\[Pl\] Stellera chamaejasme/g }
		unless ($found =~ /\[F\] strain ATCC 20928/){ $found =~ s/strain ATCC 20928/\[F\] strain ATCC 20928/g }
		unless ($found =~ /\[B\] Strept\.pulvus/){ $found =~ s/Strept\.pulvus/\[B\] Strept\.pulvus/g }
		unless ($found =~ /\[B\] Streptococcus thermophilus ST110/){ $found =~ s/Streptococcus thermophilus ST110/\[B\] Streptococcus thermophilus ST110/g }
		unless ($found =~ /\[B\] Streptomyces avermitilis NEAU1069/){ $found =~ 
			s/Streptomyces avermitilis NEAU1069/\[B\] Streptomyces avermitilis NEAU1069/g }
		unless ($found =~ /\[B\] Streptomyces griseus Acta 2871/){ $found =~ s/Streptomyces griseus Acta 2871/\[B\] Streptomyces griseus Acta 2871/g }
		unless ($found =~ /\[B\] Streptomyces griseus subsp\./){ $found =~ s/Streptomyces griseus subsp\./\[B\] Streptomyces griseus subsp\./g }
		unless ($found =~ /\[B\] Streptomyces katsunumaensis/){ $found =~ s/Streptomyces katsunumaensis/\[B\] Streptomyces katsunumaensis/g }
		unless ($found =~ /\[B\] Streptomyces sf2547/){ $found =~ s/Streptomyces sf2547/\[B\] Streptomyces sf2547/g }
		unless ($found =~ /\[B\] Streptomyces sp\./){ $found =~ s/Streptomyces sp\./\[B\] Streptomyces sp\./g }
		unless ($found =~ /\[B\] Streptomyces xanthochromogenes 6257-Mc1/){ $found =~ 
			s/Streptomyces xanthochromogenes 6257-Mc1/\[B\] Streptomyces xanthochromogenes 6257-Mc1/g }
		unless ($found =~ /\[B\] Streptoplanospora viridis/){ $found =~ s/Streptoplanospora viridis/\[B\] Streptoplanospora viridis/g }
		unless ($found =~ /\[Pl\] Symphonia globulifera/){ $found =~ s/Symphonia globulifera/\[Pl\] Symphonia globulifera/g }
		unless ($found =~ /\[Pl\] T\. aestivum/){ $found =~ s/T\. aestivum/\[Pl\] T\. aestivum/g }
		unless ($found =~ /\[F\] Talaromyces flavus IFM52668/){ $found =~ s/Talaromyces flavus IFM52668/\[F\] Talaromyces flavus IFM52668/g }
		unless ($found =~ /\[An\] Tubeastraea micrantha/){ $found =~ s/Tubeastraea micrantha/\[An\] Tubeastraea micrantha/g }
		unless ($found =~ /\[Pl\] Tulbaghia violacea/){ $found =~ s/Tulbaghia violacea/\[Pl\] Tulbaghia violacea/g }
		unless ($found =~ /\[Pl\] Ulugbekia tschimganica/){ $found =~ s/Ulugbekia tschimganica/\[Pl\] Ulugbekia tschimganica/g }
		unless ($found =~ /\[F\] Umbellomyces tetraspora/){ $found =~ s/Umbellomyces tetraspora/\[F\] Umbellomyces tetraspora/g }
		unless ($found =~ /\[F\] V\. ceratosperma/){ $found =~ s/V\. ceratosperma/\[F\] V\. ceratosperma/g }
		unless ($found =~ /\[F\] V\. rosea/){ $found =~ s/V\. rosea/\[F\] V\. rosea/g }
		unless ($found =~ /\[B\] X\. lactamgena/){ $found =~ s/X\. lactamgena/\[B\] X\. lactamgena/g }
		unless ($found =~ /\[F\] Xylaria hypoxylon AT-028/){ $found =~ s/Xylaria hypoxylon AT-028/\[F\] Xylaria hypoxylon AT-028/g }
	}
	elsif (/Isolation (.+)/){
		$found = $1;
		$field_found = "Isolation";
	}
	elsif (/C-NMR (.+)/){
		$found = $1;
		$field_found = "C-NMR";
	}
	elsif (/C-NMR-Struct (.+)/){
		$found = $1;
		$field_found = "C-NMR-Struct";
	}
	elsif (/H-NMR (.+)/){
		$found = $1;
		$field_found = "H-NMR";
	}
	elsif (/MS-Spectra (.+)/){
		$found = $1;
		$field_found = "MS-Spectra";
	}
	elsif (/UV-A (.+)/){
		$found = $1;
		$field_found = "UV-A";
	}
	elsif (/UV-B (.+)/){
		$found = $1;
		$field_found = "UV-B";
	}
	elsif (/UV-N (.+)/){
		$found = $1;
		$field_found = "UV-N";
	}
	elsif (/UV (.+)/){
		$found = $1;
		$field_found = "UV";
	}
	elsif (/IR-Spectra (.+)/){
		$found = $1;
		$field_found = "IR-Spectra";
	}
	elsif (/Toxicity (.+)/){
		$found = $1;
		$field_found = "Toxicity";
	}
	elsif (/Solubility (.+)/){
		$found = $1;
		$field_found = "Solubility";
	}
	elsif (/Activity (.+)/){
		$found = $1;
		$field_found = "Activity";
	}	
	elsif (/Appearance (.+)/){
		$found = $1;
		$field_found = "Appearance";
	}	
	elsif (/MeltingP (.+)/){
		$found = $1;
		$field_found = "MeltingP";
	}
	elsif (/TLC (.+)/){
		$found = $1;
		$field_found = "TLC";
	}
	elsif (/StructCalc (.+)/){
		$found = $1;
		$field_found = "StructCalc";
	}
	elsif (/Group (.+)/){
		$found = $1;
		$field_found = "Group";
	}
	elsif (/Remarks (.+)/){
		$found = $1;
		$field_found = "Remarks";
	}
	elsif (/References (.+)/){
		$found = $1;
		$field_found = "References";
	}
	elsif (/CA (.+)/){
		$found = $1;
		$field_found = "CA";
	}
	elsif (/REG (.+)/){
		$found = $1;
		$field_found = "REG";
	}
	elsif (/ChemClass (.+)/){
		$found = $1;
		$field_found = "ChemClass";
	}
	elsif (/Opt-Rot (.+)/){
		$found = $1;
		$field_found = "Opt-Rot";
	}
	else {
		$line_found = $_;
	}
	next if (!$id or $id < 2);
	if ($found){
		if (!$output_hash{$id}{$field_found}){ $output_hash{$id}{$field_found} = $found }
		else { $output_hash{$id}{$field_found} = $output_hash{$id}{$field_found}."; ".$found }
		$found = '';
	}
	elsif ($line_found){
		if (!$output_hash{$id}{$field_found}){ $output_hash{$id}{$field_found} = $line_found }
		else { $output_hash{$id}{$field_found} = $output_hash{$id}{$field_found}.$line_found }
		$line_found = '';
	}
}

######################################################################################################################
# Creates output file
######################################################################################################################

unless ($options{quiet}){
	print "Creating output file...\n";
}

open (OUTFILE, ">$options{outfile}") or die "Cannot open $options{outfile}";

foreach $field (@fields){
	print OUTFILE "\t$field";
}
print OUTFILE "\n";

for $counter (0..$id){
	next unless ($output_hash{$counter});
	print OUTFILE $counter;
	foreach $field (@fields){
		if (!$output_hash{$counter}{$field}){ print OUTFILE "\tno_data" }
		else { print OUTFILE "\t$output_hash{$counter}{$field}" }
	}
	print OUTFILE "\n";
}
