#!/usr/bin/perl
#
# Jonathan Klassen
# v1.31: October 17, 2012
#
# Fraction_antibase_searcher.pl
#
# Script for searching LC-MS runs containing fractionation data vs the Antibase compound database
#
# See the associated README file for copyright license.

use warnings;
use Getopt::Long;

######################################################################################################################
# Processes input arguments
######################################################################################################################

$usage = "
Fraction_antibase_searcher.pl: v1.31

USAGE:
-a	column in the input mass list containing the masses (counting from 1) 	DEFAULT: 1	e.g., perl Fraction_antibase_searcher.pl -i masses.list -a 8
-b	number of decimal places included in the mass list >=1		DEFAULT: 3		e.g., perl Fraction_antibase_searcher.pl -i masses.list -b 2
-c	which Antibase fields to display, in the format \"<field-1>, ..., <field-n>\" - see below for allowable fields
		DEFAULT: \"Name, Sources, Activity, StructCalc\"	e.g., Fraction_antibase_searcher.pl -i masses.list -c \"Name, StructCalc\"

-d	input Antibase table file					
		DEFAULT: Antibase.table					e.g., perl Fraction_antibase_searcher.pl -i masses.list -d other.table

-e	which Antibase taxa to display, in the format \"<taxon-1>, ..., <taxon-n>\" - see below for allowable taxon names
		DEFAULT: \"All\"					e.g., perl Fraction_antibase_searcher.pl -i masses.list -e \"B, C, M\"

-f	mass ions to consider, in the format \"<ion-1>, ..., <ion-n>\" - see below for allowable ion names
		DEFAULT: \"M+, MH+, MNa+\"					e.g., perl Fraction_antibase_searcher.pl -i masses.list -f \"M-, MH-\"

-g	output file of masses present in each fraction			
		DEFAULT: Fraction_IDs.table				e.g., perl Fraction_antibase_searcher.pl -i masses.list -g Something.out

-h	displays this usage statement (also using --help)					e.g., perl Fraction_antibase_searcher.pl -h
-i 	input tab-delimited file containing the molecular mass list	REQUIRED		e.g., perl Fraction_antibase_searcher.pl -i masses.list
-j 	applied error ranges, in the format \"<starting at this mass-1>,<+/- mass units-1> ... <starting at this mass-n>,<+/- mass units-n>\"
		DEFAULT: \"0,0.002 500,0.004\"				e.g., perl Fraction_antibase_searcher.pl -i masses.list -j \"0,0\"

-k	does the input molecular mass list contain a table header?	DEFAULT: Y		e.g., perl Fraction_antibase_searcher.pl -i masses.list -k N
-l	dead time during which fractions are not collected (in minutes)	DEFAULT: 0		e.g., perl Fraction_antibase_searcher.pl -i masses.list -k 3
-m	interval for fraction collection (in minutes > 0)		DEFAULT: 1		e.g., perl Fraction_antibase_searcher.pl -i masses.list -l 0.5
-n	column in the input mass list containing the retention times (counting from 1); retention times take the format \"<number>-<number>\" (spaces between are ok)
		DEFAULT: 9						e.g., perl Fraction_antibase_searcher.pl -i masses.list -j

-o	output: input file with the Antibase matches appended		
		DEFAULT: Fraction_matches.table				e.g., perl Fraction_antibase_searcher.pl -i Antibase.flatfile -o Something.out

-q	run quietly (no printed output)								e.g., perl Fraction_antibase_searcher.pl -i Antibase.flatfile -q

OUTPUT FILES:
	Fraction_IDs.table (remaned using -g flag)			Output table of masses present in each fraction with their Antibase matches
	Fraction_matches.table (renamed using -o flag)			Outputted version of the input mass table with the Antibase matches appended

Allowable Antibase field names:
	Activity, Appearance, C-NMR, C-NMR-Struct, CA, CAS-RN, ChemClass, Formula, Group, H-NMR, IR-Spectra, Isolation, Mass, MeltingP, MS-Spectra, Name, 
	Opt-Rot, REG, References, Remarks, Solubility, Sources, StructCalc, TLC, Toxicity, UV-A, UV-B, UV-N, UV, All
	\"StructCalc\" and \"Sources\" are required defaults because Antibase_mass_searcher.pl relies on those information 

Allowable Antibase taxonomic descriptions (either words or single letter abbreviations can be used):
	A (Algae), An (Animal), B (Bacteria), C (Cyanobacteria), F (Fungi), L (Lichen), M (Myxobacteria), P (Protist), Pl (Plant), S (Slime mold), 
	Y (Yeast), All (All organisms)

Allowable mass ions to consider:
	M+, M-, MH+, MH-, MNa+
";

# input arguments

%options = ();
GetOptions(
	"a=i" => \$options{mass_column},		"b=i" => \$options{decimal_places},
	"c=s" => \$options{antibase_fields},		"d=s" => \$options{antibase_input_table},
	"e=s" => \$options{antibase_taxa},		"f=s" => \$options{mass_ions},
	"g=s" => \$options{output_IDs},			"h" => \$options{help},
	"help" => \$options{help},			"i=s" => \$options{infile},
	"j=s" => \$options{error_ranges},		"k=s" => \$options{input_file_header},	
	"l=s" => \$options{dead_time},			"m=s" => \$options{fraction_length},
	"n=i" => \$options{rt_column},			"o=s" => \$options{output_matches},
	"q" => \$options{quiet}
);

# display usage statement if called

die $usage if ($options{help});

# required arguments

die $usage if (!$options{infile});

# default arguments

$options{mass_column} = 8 if (!$options{mass_column});
$options{decimal_places} = 3 if (!$options{decimal_places});
$options{antibase_fields} = "Name, Sources, Activity, StructCalc" if (!$options{antibase_fields});
$options{antibase_input_table} = "Antibase.table" if (!$options{antibase_input_table});
$options{antibase_taxa} = "All" if (!$options{antibase_taxa});
$options{mass_ions} = "M+, MH+, MNa+" if (!$options{mass_ions});
$options{output_IDs} = "Fraction_IDs.table" if (!$options{output_IDs});
$options{error_ranges} = "0,0.002 500,0.004" if (!$options{error_ranges});
$options{input_file_header} = "Y" if (!$options{input_file_header});
$options{dead_time} = 0 if (!$options{dead_time});
$options{fraction_length} = 1 if (!$options{fraction_length});
$options{rt_column} = 9 if (!$options{rt_column});
$options{output_matches} = "Fraction_matches.table" if (!$options{output_matches});

# mystery input flags not allowed

die "ERROR: Unrecognized command line arguments: @ARGV\n" if ($ARGV[0]);

# checks that input files exist

unless (-e $options{infile}){ die "ERROR: $options{infile} does not exist\n$usage" }
unless (-e $options{antibase_input_table}){ die "ERROR: $options{antibase_input_table} does not exist\n$usage" }
unless ($options{quiet}){
	unless (-e $options{output_IDs}){ print "WARNING: $options{output_IDs} exists and has been overwritten\n" }
	unless (-e $options{output_matches}){ print "WARNING: $options{output_matches} exists and has been overwritten\n" }
}

# checks for valid Antibase fields; hash keyed to order in input table

%antibase_fields = (
	'CAS-RN' => 1,		Name => 2, 		Formula => 3,		Mass => 4,
	Sources => 5,		Isolation => 6,		'C-NMR' => 7,		'C-NMR-Struct' => 8,
	'H-NMR' => 9,		'MS-Spectra' => 10, 	'UV-A' => 11, 		'UV-B' => 12,
	'UV-N' => 13,		UV => 14, 		'IR-Spectra' => 15,	Toxicity => 16,
	Solubility => 17,	Activity => 18, 	Appearance => 19,	MeltingP => 20, 
	TLC => 21, 		StructCalc => 22,	Group => 23, 		Remarks => 24,
	Refereces => 25,	CA => 26, 		REG => 27,		ChemClass => 28,
	'Opt-Rot' => 29
);

$options{antibase_fields} =~ s/\"//g;
die "ERROR: Antibase fields are formatted incorrectly\n$usage" unless ($options{antibase_fields} =~ /\w+/);
@input_antibase_fields = split ", ", $options{antibase_fields};
foreach (@input_antibase_fields){
	if ($_ eq "All" or $_ eq "all"){
		@input_antibase_fields = keys %antibase_fields;
		last;
	}
	die "ERROR: $_ is not a recognized Antibase field\n$usage" unless ($antibase_fields{$_});
}
unless (grep { /StructCalc/ } @input_antibase_fields) { push @input_antibase_fields, "StructCalc" }
unless (grep { /Sources/ } @input_antibase_fields) { push @input_antibase_fields, "Sources" }

# checks for valid Antibase taxa

%antibase_taxon_abbreviations = (
	A => Algae,		An => Animal,		B => Bacteria,
	C => Cyanobacteria,	F => Fungus,		L => Lichen, 
	M => Myxobacteria,	P => Protist,		Pl => Plant, 
	S => "Slime mold",	Y => Yeast
);

foreach (keys %antibase_taxon_abbreviations){
	$antibase_taxon_names{$antibase_taxon_abbreviations{$_}} = $_;
}

$options{antibase_taxa} =~ s/\"//g;
die "ERROR: Antibase taxa are formatted incorrectly\n$usage" unless ($options{antibase_taxa} =~ /\w+/);
@temp = split ", ", $options{antibase_taxa};
foreach (@temp){
	if ($_ eq "All" or $_ eq "all" or $_ eq "All organisms" or $_ eq "all organisms"){
		@input_antibase_taxa = keys %antibase_taxon_abbreviations;
		last;
	}
	
	if ($antibase_taxon_abbreviations{$_}){ push @input_antibase_taxa, $_ }
	elsif ($antibase_taxon_names{$_}){ push @input_antibase_taxa, $antibase_taxon_names{$_} }
	else { die "ERROR: $_ is not a recognized Antibase field\n$usage" unless ($antibase_taxon_abbreviations{$_} or $antibase_taxon_names) }
}

# checks for valid mass ions

$mass_ions{'M+'} = $mass_ions{'M-'} = $mass_ions{'MH+'} = $mass_ions{'MH-'} = $mass_ions{'MNa+'} = "Y";
%add_to_masses = (
	'M+' => -0.000549,
	'M-' => 0.000549,
	'MH+' => 1.007276,
	'MH-' => 1.008374,
	'MNa+' => 22.98922
);

$options{mass_ions} =~ s/\"//g;
die "ERROR: Antibase taxa are formatted incorrectly\n$usage" unless ($options{mass_ions} =~ /\w+/);
@input_mass_ions = split ", ", $options{mass_ions};
foreach (@input_mass_ions){ die "$_ is not a recognized mass ion\n$usage" unless ($mass_ions{$_}) }

# checks if the number of input decimal places exceeds that for the reference ions

if ($options{decimal_places} > 6){ print "WARNING: Number of input decimal places exceeds maximum; rounding to 6 places\n" }

# checks that error ranges are interpretable, rounds as needed and sets arrays

@input_error_ranges = split " ", $options{error_ranges};
foreach $input (@input_error_ranges){
	die "ERROR: Error ranges are formatted incorrectly\n$usage" unless ($input =~ /((\d|\.)+),((\d|\.)+)/);
	$start = $1;
	$error = $3;
	$decimal_place_corrector = 10 ** $options{decimal_places};
	$error = sprintf ("%.$options{decimal_places}f", int(($error * $decimal_place_corrector) + 0.5) / $decimal_place_corrector);
	$error_range{$start} = $error;
}

# print parameters unless -q flag selected

print "
----------------------------------------------------------------
Fraction_antibase_searcher.pl	Jonathan Klassen 	v1.31: October 17, 2012

parameters used:
	-a (column in the input mass list containing the masses, counting from 1) = $options{mass_column}
	-b (number of decimal places included in the mass list) = $options{decimal_places}
	-c (which Antibase fields to display, in the format \"<field-1>, ..., <field-n>\") = @input_antibase_fields
	-d (input Antibase table file) = $options{antibase_input_table}
	-e (which Antibase taxa to display, in the format \"<taxon-1>, ..., <taxon-n>\") = @input_antibase_taxa
	-f (mass ions to consider, in the format \"<ion-1>, ..., <ion-n>\") = @input_mass_ions
	-g (output file of masses present in each fraction) = $options{output_IDs}
	-i (input tab-delimited file containing the molecular mass list) = $options{infile}
	-j (applied error ranges, in the format \"<starting at this mass>,<+/- mass units>\" = $options{error_ranges}
	-k (does the input molecular mass list contain a table header?) = $options{input_file_header}
	-l (dead time during which fractions are not collected, in minutes) = $options{dead_time}
	-m (interval for fraction collection, in minutes > 0) = $options{fraction_length}
	-n (column in the input mass list containing the retention times, counting from 1) = $options{rt_column}
	-o (output: input file with the Antibase matches appended) = $options{output_matches}
----------------------------------------------------------------
" unless ($options{quiet}); 

######################################################################################################################
# reads through the antibase table file and stores everything as a hash
# first key is the antibase db id, second is each db field type
######################################################################################################################

open (INANTIBASE, $options{antibase_input_table}) or die "Cannot open $options{antibase_input_table}";

unless ($options{quiet}){ print "Parsing input antibase database...\n" }
$counter = 0;
while (<INANTIBASE>){
	chomp;
	unless ($counter == 0){ # skips table header line
		$counter = 1;
		next;
	}

	# creates hash of the antibase db

	@antibase_line = split "\t";

	# skips db entries without structural information

	next if ($antibase_line[22] eq "no_data"); 
	$antibase_line[22] =~ s/ \.\.\..*$//;

	# skips db entries that do not have organism codes matching those in the inputs

	$next_counter = 0; 
	foreach (@input_antibase_taxa){
		if ($_ eq "all"){
			$next_counter = 1;
		}
		elsif ($antibase_line[5] =~ /(\[$_\])/){
			$next_counter = 1;
		}
	}
	next unless ($next_counter == 1);

	# Determines which fields to include in the output file

	foreach $input_antibase_field (@input_antibase_fields){
		$antibase_hash{$antibase_line[0]}{$input_antibase_field} = $antibase_line[$antibase_fields{$input_antibase_field}];
	}

	# creates a hash indexing an array of ids by their mass

		# rounding logic: 
		# (i) multiply by 10^decimal_places to bring the decimal place to be rounded adjacent to the decimal point
		# (ii) add 0.5 so if above the rounding threshold the digit to the left of the decimal is incremented
		# (iii) int reduces to the previous integer
		# (iv) divide by 10^decimal_places to get the decimals back

	$decimal_place_corrector = 10 ** $options{decimal_places};

	foreach $input_mass_ion (@input_mass_ions){
		$temp = $antibase_line[22] + $add_to_masses{$input_mass_ion};
		$a = sprintf ("%.$options{decimal_places}f", int(($temp * $decimal_place_corrector) + 0.5) / $decimal_place_corrector);
		push @{$antibase_ions{$input_mass_ion}{$a}}, $antibase_line[0];
	}
}

######################################################################################################################
# Parses input peak table
# outputs output_matches table simultaineously
######################################################################################################################

open (INFILE, $options{infile}) or die "Cannot open $options{infile}";
open (OUTFILE, ">$options{output_matches}") or die "Cannot open $options{output_matches}";

unless($options{quiet}){ print "Matching masses and printing output file...\n" }

$header_counter = 0;
while (<INFILE>){
	chomp;
	@line = split "\t";

	# outputs input table header

	if ($header_counter == 0){
		if ($options{input_file_header} eq "Y" or $options{input_file_header} eq "y"){
			print OUTFILE join "\t", @line, "\t";
		}
		else {
			foreach (@line){ print OUTFILE "\t" }
		}
		print OUTFILE "Fractions_containing_mass\t";
		print OUTFILE "Antibase_matches key: hit_counter|ion_type"; 
		foreach (@input_antibase_fields){ print OUTFILE "|", $_ }
		print OUTFILE "\n";
		$header_counter = 1;
		next;
	}

	# lines are skipped unless a mass number can be recognized, e.g., to allow for column headers

	next unless ($line[$options{mass_column}-1] =~ /((\d|.)+)/);
	$mass = $1;

	# lines are skipped unless a retention time range can be recognized, e.g., to allow for column headers
	# otherwise identify rt ranges
	# lines are also skipped if the ending range occurs before fraction collection starts

	next unless ($line[$options{rt_column}-1] =~ /((\d|\.)+)\s*-\s*((\d|\.)+)/);
	$starting_rt = $1;
	$ending_rt = $3;
	next unless ($ending_rt >= $options{dead_time});

	# mass is rounded to the appropriate number of decimal places, <= 6

	if ($options{decimal_places} > 6){
		$mass = sprintf ("%.6f", int(($mass * 1000000) + 0.5) / 1000000);
	}
	else {
		$decimal_place_corrector = 10 ** $options{decimal_places};
		$mass = sprintf ("%.$options{decimal_places}f", int(($mass * $decimal_place_corrector) + 0.5) / $decimal_place_corrector);
	}

	# outputs input line with masses rounded appropriately

	for (0..$#line){
		if ($_ == $options{mass_column} - 1){
			print OUTFILE "$mass\t";
		}
		else {
			print OUTFILE "$line[$_]\t";
		}
	}

	# determines fractions in which the compound is found


	@fractions = ();
	for ($a = $starting_rt; $a < $ending_rt; $a += $options{fraction_length}){
		next if ($a < $options{dead_time});
		push @fractions, (int(($a - $options{dead_time}) / $options{fraction_length}) + 1);
	}
	if (!$fractions[0] or (int(($ending_rt - $options{dead_time}) / $options{fraction_length}) + 1) ne $fractions[$#fractions]){
		push @fractions, (int(($ending_rt - $options{dead_time}) / $options{fraction_length}) + 1);
	}

	for $a (0..$#fractions){
		unless ($a == $#fractions){
			print OUTFILE "$fractions[$a],";
		}
		else {
			print OUTFILE "$fractions[$a]\t";
		}
	}
	
	# determines tolerance for this mass

	$tolerance = '';
	foreach (sort {$a <=> $b} keys %error_range){
		if ($mass > $_){
			$tolerance = $error_range{$_};
		}
	}
	die unless ($tolerance);

	# finds matching Antibase matches

	$found_mass = '';
	%temp_found = ();
	for ($a = $mass - $tolerance; $a <= $mass + $tolerance; $a += (1 / 10 ** $options{decimal_places})){
		$a = sprintf("%.$options{decimal_places}f", $a);
		foreach $input_mass_ion (@input_mass_ions){
			next if (!$antibase_ions{$input_mass_ion}{$a});
			foreach $id (sort @{$antibase_ions{$input_mass_ion}{$a}}){
				$found_mass = "y";
				push @{$temp_found{$input_mass_ion}}, $id;
				foreach (@fractions){ # for making fraction IDs output file
					push @{$fraction_ID_table{$_}{$mass}{$input_mass_ion}}, $id;
					push @{$fraction_ID_rt_table{$_}{$mass}{$input_mass_ion}}, $line[$options{rt_column}-1];
				}
			}
		}
	}
	unless ($found_mass eq "y"){
		foreach (@fractions){
			push @{$fraction_ID_table{$_}{$mass}{"no mass found"}}, "-";
			push @{$fraction_ID_rt_table{$_}{$mass}{"no mass found"}}, $line[$options{rt_column}-1];
		}
	}

	# outputs found matches, ordered by mass ion type and molecular mass

	$counter = 0;
	foreach $input_mass_ion (@input_mass_ions){
		foreach $id (@{$temp_found{$input_mass_ion}}){
			$counter++;
			print OUTFILE "[hit_$counter]|$input_mass_ion";
			foreach $field (@input_antibase_fields){
				print OUTFILE "|$antibase_hash{$id}{$field}";
			}
			print OUTFILE " -- ";
		}
	}
	print OUTFILE "\n";
}

######################################################################################################################
# Outputs output_IDs table 
######################################################################################################################

unless($options{quiet}){ print "Constructing fraction ID output file...\n" }

open (OUTFILE, ">$options{output_IDs}") or die "Cannot open $options{output_IDs}";
print OUTFILE "Fraction\tmeasured_mass\tretention_time\tmass_ion";
foreach $field (@input_antibase_fields){ print OUTFILE "\t$field" }
print OUTFILE "\n";

%found_masses = ();
foreach $fraction (sort {$a <=> $b} keys %fraction_ID_table){
	foreach $mass (sort {$a <=> $b} keys %{$fraction_ID_table{$fraction}}){
		foreach $input_mass_ion (@input_mass_ions){
			for $id (0..$#{$fraction_ID_table{$fraction}{$mass}{$input_mass_ion}}){
				$found_masses{$fraction}{$mass} = "y";
				print OUTFILE "$fraction\t$mass\t$fraction_ID_rt_table{$fraction}{$mass}{$input_mass_ion}[$id]\t$input_mass_ion";
				foreach $field (@input_antibase_fields){
					print OUTFILE "\t$antibase_hash{$fraction_ID_table{$fraction}{$mass}{$input_mass_ion}[$id]}{$field}";
				}
				print OUTFILE "\n";
			}
		}
		if ($fraction_ID_table{$fraction}{$mass}{"no mass found"}){
			print OUTFILE "$fraction\t$mass\t", $fraction_ID_rt_table{$fraction}{$mass}{"no mass found"}[0], "\tno Antibase match found\n";
		}
	}
}
#	push @{$fraction_ID_rt_table{$_}{$mass}{$input_mass_ion}}, $line[$options{rt_column}-1];
